from __future__ import absolute_import as _

import _cffi_backend as _

from .machine import LikNormMachine
from .testit import test

__name__ = "liknorm"
__version__ = "1.0.12"
__author__ = "Danilo Horta"
__author_email__ = "horta@ebi.ac.uk"

__all__ = [
    "__name__", "__version__", "__author__", "__author_email__", "test",
    "LikNormMachine"
]
